Exception: StatsD::Instrument::CompiledMetric::DefinitionError
- Inherits:
-
StandardError
- Object
- StandardError
- StatsD::Instrument::CompiledMetric::DefinitionError
- Defined in:
- lib/statsd/instrument/compiled_metric.rb
Overview
Raised when a CompiledMetric subclass is defined incorrectly.
This includes calling define on a base type class directly,
calling define more than once, or using a metric before define
has been called.